@@ -6,6 +6,8 @@ from pathlib import Path
 import copy
 import os
 import re
+import shutil
+import sys
 import time
 from logging import getLogger, Formatter, StreamHandler
 from typing import Any, List, Optional, Tuple
@@ -423,6 +425,11 @@ class DicomFile(Data):
         except IsADirectoryError:
             self.ds = None
 
+        # Try to protect against cases where file read isn't a DICOM file.
+        if (not isinstance(self.ds, pydicom.dataset.FileDataset)
+                or len(self.ds) < 2):
+            self.ds = None
+
         # Define prefixes of dataset attributes to be used
         # to set study and item timestamps.
         elements = {
@@ -816,6 +823,107 @@ class Dated(PathData):
         '''
         return (self > other) or (self == other)
 
+    def copy_dicom(self, outdir=None, overwrite=True, sort=True):
+        """
+        Copy (single) source dicom file.
+
+        Derived classes may override this method, to provide for
+        copying multiple files.  See, for example:
+            skrt.image.Image.copy_dicom().
+
+        **Parameters:**
+
+        outdir : pathlib.Path/str, default=None
+            Path to directory to which source file is to be copied.
+            if None, this is set to be a directory with name equal
+            to the modality (all upper case) of the dicom data.
+
+        overwrite : bool, default=True
+            If True, delete and recreate <outdir> before copying
+            file.  If False and <outdir> exists already, copy
+            file only if this doesn't mean overwriting an existing
+            file.
+
+        sort : bool, default=True
+            If True, the copied dicom file will be given name of form
+            'MODALITY_YYMMDD_hhmmss'.  If False, the file is copied
+            with name unaltered.
+        """
+        if not callable(getattr(self, "load", None)):
+            raise NotImplementedError(
+                    f"{type(self)}.copy_dicom() failed - "
+                    "class has no load() method")
+            return
+        self.load()
+
+        # Check that object has dicom file to be copied.
+        path = Path(self.path)
+        ds = getattr(self, "dicom_dataset", None)
+        if not ds or not path.exists():
+            raise NotImplementedError(
+                    f"{type(self)}.copy_dicom() failed - "
+                    "object has no associated DICOM file")
+            return
+
+        # Obtain the data modality.
+        modality = getattr(ds, "Modality", "unknown")
+
+        # Define the output directory.
+        outdir = make_dir(outdir or modality, overwrite)
+
+        # Define path to the output file.
+        if sort:
+            name = f"{modality}_{self.timestamp}.dcm" if sort else path.name()
+        outpath = outdir / name
+
+        # Copy file.
+        if overwrite or not outpath.exists():
+            shutil.copy2(path, outpath)
+
+    def copy_dicom_files(self, data_type=None, index=None, indices=None,
+            outdir="dicom", overwrite=True, sort=True):
+        """
+        Copy DICOM file(s) associated with this object.
+
+        **Parameters:**
+
+        data_type : str, default=None
+            String indicting type of data associated with
+            this object.  This may be any key of the <indices>
+            dictionary.  Ignored if not a key of this dictionary,
+            or if None.
+
+        index : int, default=None
+            Integer representing object position in an ordered
+            list of objects of specified <data_type>.  Ignored if None.
+
+        indices : dict, default=None
+            Dictionary where the keys are data types and the values are
+            lists of indices for the objects whose data is to be written.
+            Ignored if None.
+
+        outdir : pathlib.Path/str, default="dicom"
+            Path to directory to which dicom files are to be copied.
+
+        overwrite : bool, default=True
+            If True, allow copied file(s) to overwrite existing file(s),
+            as defined in derived class's copy_dicom() method.
+
+        sort : bool, default=True
+            If True, files are sorted for copying, as defined in derived
+            class's copy_dicom() method.
+        """
+        # Check whether object index satisfies requirements.
+        if (isinstance(indices, dict)
+                and isinstance(data_type, str)
+                and isinstance(indices.get(data_type, None), list)
+                and isinstance(data_index, int)
+                and data_index not in indices[data_type]):
+            return
+
+        # Create output directory, and copy object files.
+        outdir = make_dir(outdir, overwrite)
+        self.copy_dicom(outdir, overwrite, sort)
 
 class MachineData(Dated):
     """Dated object with an associated machine name."""
@@ -956,7 +1064,7 @@ def get_logger(name="", log_level=None, identifier="name"):
         https://docs.python.org/3/library/logging.html#logrecord-attributes
     """
     formatter = Formatter(f"%({identifier})s - %(levelname)s - %(message)s")
-    handler = StreamHandler()
+    handler = StreamHandler(sys.stdout)
     handler.setFormatter(formatter)
     logger = getLogger(name)
     if not logger.handlers:
@@ -1481,3 +1589,166 @@ def relative_path(path, nlevel=None):
     elements = fullpath(path).split(os.sep)
     idx = min(nlevel + 1, len(elements) - 1) if nlevel > 0 else nlevel
     return os.sep.join(elements[idx:])
+
+def get_indexed_objs(objs, indices=True):
+    """
+    Select subset of objects from a list.
+
+    **Parameters:**
+    objs : list
+        List of objects.
+
+    indices : bool/list/int
+        Specification of objects to select from list:
+
+        - True: select all objects;
+        - False: select no objects;
+        - integer: select object with index equal to given integer;
+        - list of integers: select objects with indices equal to given integers.
+    """
+
+    # Select all objects.
+    if indices is True:
+        indexed_objs = objs
+    # Select no objects.
+    elif indices is False:
+        indexed_objs = []
+    # Select object with specified index.
+    elif isinstance(indices, int):
+        indexed_objs = [objs[indices]]
+    # Select objects with specified indices.
+    else:
+        indexed_objs = [objs[idx] for idx in indices]
+
+    return indexed_objs
+
+def make_dir(path=".", overwrite=True, require_empty=False):
+    """
+    Create a directory if it doesn't exist already, or if overwriting allowed.
+
+    **Parameters:**
+
+    path : pathlib.Path/str, default="."
+        Path to directory.
+
+    overwrite : bool, default=True
+        If True, delete any pre-existing directory and its contents.
+        If False, leave unaltered any pre-existing directory and its contents.
+
+    require_empty : bool, default=False
+        If True, return None if the directory already exists, the directory
+        isn't empty, and <overwrite> is False.
+    """
+    # Obtain pathlib.Path object for directory.
+    dirpath = Path(fullpath(path))
+
+    if dirpath.exists():
+        # Directory exists, and should be overwritten.
+        if overwrite:
+            shutil.rmtree(dirpath)
+        # Directory exists, and isn't empty.
+        elif require_empty and any(dirpath.iterdir()):
+            dirpath = None
+
+    if dirpath is not None:
+        # Create directory if needed.
+        dirpath.mkdir(parents=True, exist_ok=True)
+    
+    return dirpath
+
+def get_float(obj, attribute, default=None):
+    """
+    Return object attribute as a float.
+
+    **Parameters:**
+
+    obj : Object to be considered.
+
+    attribute : str
+        Name of object attribute to be returned as float.
+
+    default : Value to be returned in case object attribute
+        can't be converted to float.
+    """
+    value = getattr(obj, attribute, default)
+    if value != default:
+        try:
+            value = float(value)
+        except (TypeError, ValueError):
+            value = default
+    return value
+
+def get_qualified_class_name(cls):
+    """
+    Determine qualified name of class <cls>.
+    """
+    if isinstance(cls, type):
+        return f"{cls.__module__}.{cls.__name__}"
+
+def get_subdir_paths(parent):
+    """
+    Return paths to a directory's sub-directories.
+
+    **Parameter:**
+
+    parent : pathlib.Path/str
+        Path to directory for which sub-directory paths are to be found.
+    """
+    return sorted([subdir for subdir in Path(parent).iterdir()
+        if subdir.is_dir()])
+
+def get_data_indices(in_value, valid_data_types):
+    """
+    Obtain dictionary associating indices with data types.
+
+    This is used in patient.skrt.Study.copy_dicom() to obtain
+    dictionaries from input parameters that may be dictionaries,
+    lists, or None.
+
+    **Parameters:**
+
+    in_value : dict/str/int/list/None
+        Specification of indices associated with data types.
+        list of data types, or None.  If a dictionary, return
+        after removing any keys not in <valid_data_types>.
+        If a string, return a dictionary that associates with
+        it the value True (all file indices) if the string is
+        in <valid_data_types>, or otherwise return an empty
+        dictionary.  If an integer, return a dictionary that associates
+        this value with all data types of <valid_data_types>.  If a
+        list, return a dictionary that associates the value True with
+        each listed data type that is in <valid_data_types>.  If None,
+        return a dictionary that associates the value True with all
+        data types of <valid_data_types>.
+
+    valid_data_types : list
+        List of valid data_types.
+    """
+    # Filter out keys that aren't valid data types.
+    if isinstance(in_value, dict):
+        data_indices = {data_type: indices
+                for data_type, indices in in_value.items()
+                if data_type in valid_data_types}
+
+    # Associate specific index with all valid data types.
+    elif isinstance(in_value, int):
+        data_indices = {data_type: in_value for data_type in valid_data_types}
+
+    # Associate value True with specific data type.
+    elif isinstance(in_value, str):
+        data_indices = {in_value: True} if in_value in valid_data_types else {}
+
+    else:
+
+        # Accept all valid data types.
+        if in_value is None:
+            data_types = valid_data_types
+
+        # Accept subset of valid data types.
+        else:
+            data_types = [data_type for data_type in in_value
+                    if data_type in valid_data_types]
+
+        data_indices = {data_type: True for data_type in data_types}
+
+    return data_indices
